<?php

class BWFAN_GroundHogg_Common {

	public static $is_v2 = true;
	private static $_instance = null;

	public function __construct() {

		if ( ! defined( 'GROUNDHOGG_VERSION' ) || version_compare( GROUNDHOGG_VERSION, '2.0', '<' ) ) {
			self::$is_v2 = false;
		}
	}

	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
	}

	/**
	 *  get custom fields of contact set by groundhogg
	 */
	public static function get_default_custom_fields() {
		$groundhogg_fields = array(
			'first_name'        => array(
				'crm_label' => __( 'First Name', 'autonami-automations-pro' ),
				'crm_field' => 'first_name',
			),
			'last_name'         => array(
				'crm_label' => __( 'Last Name', 'autonami-automations-pro' ),
				'crm_field' => 'last_name',
			),
			'user_email'        => array(
				'crm_label' => __( 'Email', 'autonami-automations-pro' ),
				'crm_field' => 'email',
			),
			'phone_number'      => array(
				'crm_label' => __( 'Phone', 'autonami-automations-pro' ),
				'crm_field' => 'phone',
			),
			'billing_address_1' => array(
				'crm_label' => __( 'Address 1', 'autonami-automations-pro' ),
				'crm_field' => 'street_address_1',
			),
			'billing_address_2' => array(
				'crm_label' => __( 'Address 2', 'autonami-automations-pro' ),
				'crm_field' => 'street_address_2',
			),
			'billing_city'      => array(
				'crm_label' => __( 'City', 'autonami-automations-pro' ),
				'crm_field' => 'city',
			),
			'billing_state'     => array(
				'crm_label' => __( 'State', 'autonami-automations-pro' ),
				'crm_field' => 'region',
			),
			'billing_postcode'  => array(
				'crm_label' => __( 'Zip', 'autonami-automations-pro' ),
				'crm_field' => 'postal_zip',
			),
			'billing_country'   => array(
				'crm_label' => __( 'Country', 'autonami-automations-pro' ),
				'crm_field' => 'country',
			),
			'company'           => array(
				'crm_label' => __( 'Company', 'autonami-automations-pro' ),
				'crm_field' => 'company',
			),
			'website'           => array(
				'crm_label' => __( 'Website', 'autonami-automations-pro' ),
				'crm_field' => 'website',
			),
			'user_url'          => array(
				'crm_label' => __( 'User Url', 'autonami-automations-pro' ),
				'crm_field' => 'website',
			),
			'gdpr_consent'      => array(
				'crm_label' => __( 'GDPR Consent', 'autonami-automations-pro' ),
				'crm_field' => 'gdpr_consent'
			),
			'terms_agreement'   => array(
				'crm_label' => __( 'Agreed to Terms', 'autonami-automations-pro' ),
				'crm_field' => 'terms_agreement'
			),
		);

		return $groundhogg_fields;
	}

	/** get all available tags
	 * @return array
	 */
	public static function get_tags() {
		$available_tags = array();

		if ( self::$is_v2 ) {

			$data = \Groundhogg\Plugin::$instance->dbs->get_db( 'tags' )->search();

		} else {

			$data = WPGH()->tags->get_tags();

		}

		foreach ( $data as $row ) {
			$available_tags[ $row->tag_id ] = $row->tag_name;
		}

		return $available_tags;
	}


	/**
	 * Loads all custom fields from CRM and merges with local list
	 *
	 * @access public
	 * @return array CRM Fields
	 */

	public static function get_fields() {

		$crm_fields = array();

		if ( self::$is_v2 ) {

			$groundhogg_fields = BWFAN_GroundHogg_Common::get_default_custom_fields();
			foreach ( $groundhogg_fields as $field ) {

				$crm_fields[ $field['crm_field'] ] = $field['crm_label'];
			}
			$meta_keys = \Groundhogg\Plugin::$instance->dbs->get_db( 'contactmeta' )->get_keys();

			foreach ( $meta_keys as $meta_key ) {
				$crm_fields[ $meta_key ] = ucwords( str_replace( '_', ' ', $meta_key ) );
			}
		} else {

			$data = new WPGH_Contact( 0 );

			foreach ( $data as $key => $field_data ) {
				$crm_fields[ $key ] = ucwords( str_replace( '_', ' ', $key ) );
			}

			global $wpdb;

			$meta_keys = $wpdb->get_col( "SELECT DISTINCT meta_key FROM {$wpdb->prefix}gh_contactmeta ORDER BY meta_key DESC" );

			foreach ( $meta_keys as $meta_key ) {
				$crm_fields[ $meta_key ] = ucwords( str_replace( '_', ' ', $meta_key ) );
			}
		}

		asort( $crm_fields );
		$crm_fields = array_filter( $crm_fields );

		return $crm_fields;
	}


	/**
	 * Gets contact ID for a user based on email address
	 * @return int Contact ID
	 */

	public static function get_contact_id( $email_address ) {

		if ( self::$is_v2 ) {

			$contact = \Groundhogg\Plugin::$instance->dbs->get_db( 'contacts' )->get_contact_by( 'email', $email_address );

		} else {

			$contact = WPGH()->contacts->get_contact_by( 'email', $email_address );

		}

		if ( empty( $contact ) ) {
			return false;
		}

		return $contact->ID;

	}

	/**
	 * Update contact
	 *
	 * @access public
	 * @return bool
	 */

	public static function update_contact( $contact_id, $data, $map_meta_fields = true ) {

		if ( empty( $data ) ) {
			return false;
		}

		if ( self::$is_v2 ) {

			$contact = new \Groundhogg\Contact( $contact_id );

		} else {

			$contact = new WPGH_Contact( $contact_id );

		}

		if ( ! $contact instanceof \Groundhogg\Contact ) {
			return false;
		}

		foreach ( $data as $key => $value ) {

			$contact->update_meta( $key, $value );

		}

		return true;
	}

	/** remove tags from contact
	 *
	 * @param $tags
	 * @param $contact_id
	 *
	 * @return bool
	 */
	public static function remove_tags( $tags, $contact_id ) {

		if ( self::$is_v2 ) {
			$contact = new \Groundhogg\Contact( $contact_id );

			$contact->remove_tag( $tags );

		} else {

			$contact = new WPGH_Contact( $contact_id );

			foreach ( $tags as $key => $value ) {

				$contact->remove_tag( array( $contact_id => $value ) );

			}
		}

		return true;

	}

	/**
	 * Adds a new contact
	 *
	 * @access public
	 * @return int|array Contact ID
	 */

	public static function add_contact( $data, $map_meta_fields = false ) {

		if ( self::$is_v2 ) {


			$contact = new \Groundhogg\Contact( $data );

			if ( ! $contact->exists() ) {
				return array(
					'status'  => 4,
					'message' => __( 'Unable to Create Contact', 'autonami-automations-pro' ),
				);
			}

			$id = $contact->get_id();

		} else {


			$id = WPGH()->contacts->add( $data );

			$contact = new WPGH_Contact( $id );

		}

		foreach ( $data as $key => $value ) {

			$contact->update_meta( $key, $value );

		}

		return $id;

	}

}

BWFAN_GroundHogg_Common::get_instance();