<?php

/**
 * Plugin Name: Autonami Marketing Automations - GroundHogg Addon
 * Plugin URI: https://buildwoofunnels.com
 * Description: Now create GroundHogg based automations with Autonami Marketing Automations
 * Version: 1.0.0
 * Author: WooFunnels
 * Author URI: https://buildwoofunnels.com
 * License: GPLv3 or later
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain: autonami-automations-pro
 *
 * Requires at least: 4.9
 * Tested up to: 5.3
 */

if ( ! defined( 'GROUNDHOGG_VERSION' ) ) {
	return;
}


final class BWFAN_GroundHogg {

	private static $_instance = null;

	private function __construct() {
		add_action( 'bwfan_loaded', [ $this, 'init_pro' ] );
		add_action( 'bwfan_before_automations_loaded', [ $this, 'add_modules' ] );
	}

	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
	}

	public function init_pro() {
		$this->define_plugin_properties();
		$this->core();
	}

	public function define_plugin_properties() {
		define( 'BWFAN_GROUNDHOGG_VERSION', '1.0.0' );
		define( 'BWFAN_GROUNDHOGG_FULL_NAME', 'Autonami Marketing Automations Pro' );
		define( 'BWFAN_GROUNDHOGG_PLUGIN_FILE', __FILE__ );
		define( 'BWFAN_GROUNDHOGG_PLUGIN_DIR', __DIR__ );
		define( 'BWFAN_GROUNDHOGG_PLUGIN_URL', untrailingslashit( plugin_dir_url( BWFAN_GROUNDHOGG_PLUGIN_FILE ) ) );
		define( 'BWFAN_GROUNDHOGG_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
		define( 'BWFAN_GROUNDHOGG_IS_DEV', true );
		define( 'BWFAN_GROUNDHOGG_DB_VERSION', '1.0' );
		define( 'BWFAN_GROUNDHOGG_ENCODE', sha1( BWFAN_GROUNDHOGG_PLUGIN_BASENAME ) );
	}

	private function core() {
		require BWFAN_GROUNDHOGG_PLUGIN_DIR . '/includes/class-bwfan-groundhogg-common.php';
	}

	/**
	 *  register all the files
	 */
	public function add_modules() {
		$integration_dir = BWFAN_GROUNDHOGG_PLUGIN_DIR . '/autonami';
		foreach ( glob( $integration_dir . '/class-*.php' ) as $_field_filename ) {
			if ( strpos( $_field_filename, 'index.php' ) !== false ) {
				continue;
			}
			require_once( $_field_filename );
		}
		do_action( 'bwfan_groundhogg_integrations_loaded', $this );
	}

	/**
	 * to avoid unserialize of the current class
	 */
	public function __wakeup() {
		throw new ErrorException( 'BWFAN_Core GroundHogg can`t converted to string' );
	}

	/**
	 * to avoid serialize of the current class
	 */
	public function __sleep() {
		throw new ErrorException( 'BWFAN_Core GroundHogg can`t converted to string' );
	}

	/**
	 * To avoid cloning of current class
	 */
	protected function __clone() {
	}
}

BWFAN_GroundHogg::get_instance();